/*
 * Copyright (c) 2011, Yuri S. Zemskov All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the ASX Software nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package kernel;

import java.util.*;

/**
 * Главный класс ядра. Экземпляр этого класса обеспечивает управление модулями и
 * их взаимодействие.
 * @author Yuri S. Zemskov (Юрий Земсков)
 */
public class Kernel {

    /*
     * Таблица процессов
     * Ключом является PID, значением --- массив из имени модуля, коллекции
     * имен перехватываемых модулей и коллекции имен обрабатываемых модулей.
     */
    private Hashtable processes;
    /*
     * Таблица модулей
     * Ключом является имя модуля, значением --- массив из коллекции PID'ов
     * процессов --- экземпляров данного модуля, коллекции PID'ов процессов ---
     * перехватчиков и коллекции PID'ов процессов --- обработчиков событий от
     * данного модуля.
     */
    private Hashtable mods;
    /*
     * Номер следующего процесса.
     */
    private int nextPID = 0;

    public Kernel() {
        processes = new Hashtable();
        mods = new Hashtable();
    }

    /*
     * Добавляет процесс в таблицу процессов. Вызывается при загрузке модуля.
     */
    private void addProcessToTable(int PID, String mod) {
        processes.put(new Integer(PID), new Object[]{mod, new Vector(), new Vector()});
    }

    /*
     * Добавляет в таблицу процессов имя модуля, события которого
     * перехватываются указанным процессом. Вызывается при регистрации
     * перехватчика.
     */
    private void addInterceptedMod(int PID, String mod) {
        Vector v = (Vector) ((Object[]) processes.get(new Integer(PID)))[1];
        v.addElement(mod);
    }

    /*
     * Добавляет в таблицу процессов имя модуля, события которого
     * обрабатываются указанным процессом.Вызывается при регистрации
     * обработчика.
     */
    private void addListenedMod(int PID, String mod) {
        Vector v = (Vector) ((Object[]) processes.get(new Integer(PID)))[2];
        v.addElement(mod);
    }

    /*
     *
     */
    private void removeInterceptedMod(int PID, String mod) {
        Vector v = (Vector) ((Object[]) processes.get(new Integer(PID)))[1];
    }
}
